import argparse
import os

from autolinkedin.crons.cron_jobs import LinkedInCron
from autolinkedin.linkedin import LinkedIn
from autolinkedin.ln_settings import LinkedInCommandArgs, LinkedInSettingsName, get_linkedin_settings

parser = argparse.ArgumentParser()

parser.add_argument(
    LinkedInCommandArgs.LINKEDIN_ENV_FILE,
    type=str,
    default=os.getenv(LinkedInSettingsName.LINKEDIN_ENV_FILE, None),
    help="Linkedin environment file",
)

parser.add_argument(
    LinkedInCommandArgs.LINKEDIN_USER,
    type=str,
    default=os.getenv(LinkedInSettingsName.LINKEDIN_USER, None),
    help="Email of LinkedIn user",
)

parser.add_argument(
    LinkedInCommandArgs.LINKEDIN_PASSWORD,
    type=str,
    default=os.getenv(LinkedInSettingsName.LINKEDIN_PASSWORD, None),
    help="Password of LinkedIn user",
)

parser.add_argument(
    LinkedInCommandArgs.LINKEDIN_BROWSER,
    type=str,
    default=os.getenv(LinkedInSettingsName.LINKEDIN_BROWSER, "Chrome"),
    help="Browser used for LinkedIn",
)


parser.add_argument(
    LinkedInCommandArgs.LINKEDIN_BROWSER_HEADLESS,
    action="store_true",
    help="Whether to run headless (i.e. without the browser visible in the front.)",
    default=os.getenv(LinkedInSettingsName.LINKEDIN_BROWSER_HEADLESS, False),
)

parser.add_argument(
    LinkedInCommandArgs.LINKEDIN_MAX_INVITE,
    type=int,
    default=os.getenv(LinkedInSettingsName.LINKEDIN_MAX_INVITE, 0),
    help="Maximum number of invitations to send",
)

parser.add_argument(
    LinkedInCommandArgs.LINKEDIN_MIN_MUTUAL,
    type=int,
    default=os.getenv(LinkedInSettingsName.LINKEDIN_MIN_MUTUAL, 0),
    help="Minimum number of mutual connections required.",
)

parser.add_argument(
    LinkedInCommandArgs.LINKEDIN_MAX_MUTUAL,
    type=int,
    default=os.getenv(LinkedInSettingsName.LINKEDIN_MAX_MUTUAL, 500),
    help="Maximum number of mutual connections required.",
)

parser.add_argument(
    LinkedInCommandArgs.LINKEDIN_WITHDRAW_INVITE_BEFORE_DAYS,
    type=int,
    default=os.getenv(LinkedInSettingsName.LINKEDIN_WITHDRAW_INVITE_BEFORE_DAYS, 14),
    help="Withdraw invites older than this many days",
)


parser.add_argument(
    LinkedInCommandArgs.LINKEDIN_PREFERRED_USER,
    type=str,
    default=os.getenv(LinkedInSettingsName.LINKEDIN_PREFERRED_USER, ""),
    help="Path to file containing preferred users characteristics",
)

parser.add_argument(
    LinkedInCommandArgs.LINKEDIN_NOT_PREFERRED_USER,
    type=str,
    default=os.getenv(LinkedInSettingsName.LINKEDIN_NOT_PREFERRED_USER, ""),
    help="Path to file containing characteristics of not preferred users",
)

parser.add_argument(
    "--cronfile",
    type=str,
    default=None,
    help="Path to cronfile",
)


parser.add_argument(
    "--cronuser",
    type=str,
    default=None,
    help="Name of user setting cron on the machine (needed by most OS)",
)

parser.add_argument(
    "--rmcron",
    type=bool,
    action=argparse.BooleanOptionalAction,
    help="Whether to remove existing crons.",
)

parser.add_argument(
    "--cronhour",
    type=int,
    default=None,
    help="hour of the day you want to set cron for each day.",
)


args = parser.parse_args()

if args.rmcron:
    if not args.cronuser:
        print("You must set cronuser to delete crons")
        exit(1)

    LinkedInCron.remove_cron_jobs(args.cronuser)
    exit(0)

if args.cronfile:
    if not args.cronuser or not args.cronhour:
        print("You must set cronuser and cronhour to set crons")
        exit(1)

    LinkedInCron.set_smart_cron(cron_env_file=args.cronfile, cron_user=args.cronuser, hour_of_day=args.cronhour)

    exit(0)

settings = get_linkedin_settings(command_args=args)

if not settings.LINKEDIN_USER and not settings.LINKEDIN_PASSWORD:
    print("Please provide username and password")
    exit(1)

with LinkedIn(
    username=settings.LINKEDIN_USER,
    password=settings.LINKEDIN_PASSWORD,
    browser=settings.LINKEDIN_BROWSER,
    headless=bool(settings.LINKEDIN_BROWSER_HEADLESS),
) as ln:
    ln.smart_follow_unfollow(
        min_mutual=settings.LINKEDIN_MIN_MUTUAL,
        max_mutual=settings.LINKEDIN_MAX_MUTUAL,
        users_preferred=settings.LINKEDIN_PREFERRED_USER,
        users_not_preferred=settings.LINKEDIN_NOT_PREFERRED_USER,
        max_invitations_to_send=settings.LINKEDIN_MAX_INVITE,
        withdraw_invite_older_than_days=settings.LINKEDIN_WITHDRAW_INVITE_BEFORE_DAYS,
        remove_recommendations=True,
    )

exit(0)



import os
from dataclasses import dataclass
from pathlib import Path

from dotenv import load_dotenv


@dataclass()
class LinkedInSettings:
    LINKEDIN_ENV_FILE: str = ""
    LINKEDIN_USER: str = ""
    LINKEDIN_PASSWORD: str = ""
    LINKEDIN_BROWSER: str = "Chrome"
    LINKEDIN_BROWSER_HEADLESS: bool = False
    LINKEDIN_PREFERRED_USER: str = None
    LINKEDIN_NOT_PREFERRED_USER: str = None

    LINKEDIN_MAX_INVITE: int = 0

    LINKEDIN_MIN_MUTUAL: int = 0
    LINKEDIN_MAX_MUTUAL: int = 500

    LINKEDIN_WITHDRAW_INVITE_BEFORE_DAYS: int = 14


class LinkedInSettingsName:
    LINKEDIN_ENV_FILE = "LINKEDIN_ENV_FILE"
    LINKEDIN_USER = "LINKEDIN_USER"
    LINKEDIN_PASSWORD = "LINKEDIN_PASSWORD"  # nosec
    LINKEDIN_BROWSER = "LINKEDIN_BROWSER"
    LINKEDIN_MIN_MUTUAL = "LINKEDIN_MIN_MUTUAL"
    LINKEDIN_MAX_MUTUAL = "LINKEDIN_MAX_MUTUAL"
    LINKEDIN_BROWSER_HEADLESS = "LINKEDIN_BROWSER_HEADLESS"
    LINKEDIN_PREFERRED_USER = "LINKEDIN_PREFERRED_USER"
    LINKEDIN_NOT_PREFERRED_USER = "LINKEDIN_NOT_PREFERRED_USER"

    LINKEDIN_MAX_INVITE = "LINKEDIN_MAX_INVITE"
    LINKEDIN_WITHDRAW_INVITE_BEFORE_DAYS = "LINKEDIN_WITHDRAW_INVITE_BEFORE_DAYS"


class LinkedInCommandArgs:
    LINKEDIN_ENV_FILE = "--env"
    LINKEDIN_USER = "--email"
    LINKEDIN_PASSWORD = "--password"  # nosec
    LINKEDIN_BROWSER = "--browser"
    LINKEDIN_BROWSER_HEADLESS = "--headless"
    LINKEDIN_PREFERRED_USER = "--preferred"
    LINKEDIN_NOT_PREFERRED_USER = "--notpreferred"

    LINKEDIN_MIN_MUTUAL = "--minmutual"
    LINKEDIN_MAX_MUTUAL = "--maxmutual"

    LINKEDIN_MAX_INVITE = "--maxinvite"

    LINKEDIN_WITHDRAW_INVITE_BEFORE_DAYS = "--withdrawdays"


class LinkedInUser:
    def __init__(self, username):
        self.username = username
        self.total_followers = 0
        self.mutual_connections = 0


class LinkedInSettingsException(Exception):
    pass


def get_linkedin_settings(command_args=None) -> LinkedInSettings:
    """Get all settings or variables set by the user"""

    if env_file := getattr(command_args, LinkedInCommandArgs.LINKEDIN_ENV_FILE.strip("--"), None):
        if not Path(env_file).exists():
            raise LinkedInSettingsException("Env file does not exist. Please pass a valid env file.")
        load_dotenv(env_file)

    settings = LinkedInSettings()

    for arg_name in dir(LinkedInCommandArgs):
        if arg_name.startswith("LINKEDIN"):
            value = os.getenv(arg_name, None) or getattr(
                command_args, getattr(LinkedInCommandArgs, arg_name).strip("--")
            )

            setattr(settings, arg_name, value)

    # TODO: Manual typecasting? too bad ';(

    settings.LINKEDIN_BROWSER_HEADLESS = int(settings.LINKEDIN_BROWSER_HEADLESS)
    settings.LINKEDIN_MAX_MUTUAL = int(settings.LINKEDIN_MAX_MUTUAL)
    settings.LINKEDIN_MIN_MUTUAL = int(settings.LINKEDIN_MIN_MUTUAL)
    settings.LINKEDIN_MAX_INVITE = int(settings.LINKEDIN_MAX_INVITE)
    settings.LINKEDIN_WITHDRAW_INVITE_BEFORE_DAYS = int(settings.LINKEDIN_WITHDRAW_INVITE_BEFORE_DAYS)

    settings.LINKEDIN_PREFERRED_USER = str(Path(settings.LINKEDIN_PREFERRED_USER).absolute())
    settings.LINKEDIN_NOT_PREFERRED_USER = str(Path(settings.LINKEDIN_NOT_PREFERRED_USER).absolute())

    return settings



from contextlib import suppress
from functools import wraps


def ignore_exception(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        with suppress():
            func(*args, **kwargs)

    return wrapper



from autolinkedin.linkedin import LinkedIn

settings = {
    "LINKEDIN_USER": "",
    "LINKEDIN_PASSWORD": "",
    "LINKEDIN_BROWSER": "Chrome",
    "LINKEDIN_BROWSER_HEADLESS": 0,
    "LINKEDIN_PREFERRED_USER": "data/user_not_preferred.txt",
    "LINKEDIN_NOT_PREFERRED_USER": "data/user_preferred.txt",
}

with LinkedIn(
    username=settings.get("LINKEDIN_USER"),
    password=settings.get("LINKEDIN_PASSWORD"),
    browser=settings.get("LINKEDIN_BROWSER"),
    headless=bool(settings.get("LINKEDIN_BROWSER_HEADLESS")),
) as ln:
    ln.login()

    ln.remove_recommendations(min_mutual=0, max_mutual=50)

    max_invitations = ln.WEEKLY_MAX_INVITATION - ln.count_invitations_sent_last_week()

    print(max_invitations)

    ln.withdraw_sent_invitations(max_remove=2, older_than_days=14)

    ln.send_invitations(
        max_invitations=2,
        min_mutual=100,
        max_mutual=550,
        preferred_users="./data/user_preferred.txt",
        not_preferred_users="./data/user_not_preferred.txt",
        view_profile=True,
    )

    ln.accept_invitations()
